==> loadmychunks-1.2.0%2B1.18.2%2Bfabric-sources.jar: com/drathonix/loadmychunks/common/mixin/MixinServerLevel.java <==
package com.drathonix.loadmychunks.common.mixin;

import com.drathonix.loadmychunks.common.bridge.IChunkMapMixin;
import com.drathonix.loadmychunks.common.bridge.ILevelChunkMixin;
import com.drathonix.loadmychunks.common.bridge.IServerLevelMixin;
import com.drathonix.loadmychunks.common.system.ChunkDataManager;
//? if >1.18.1 {
//?}
import it.unimi.dsi.fastutil.longs.Long2ObjectLinkedOpenHashMap;
import it.unimi.dsi.fastutil.objects.ObjectIterator;
import it.unimi.dsi.fastutil.objects.ObjectSet;
import net.minecraft.class_1297;
import net.minecraft.class_26;
import net.minecraft.class_2794;
import net.minecraft.class_3193;
import net.minecraft.class_32;
import net.minecraft.class_3215;
import net.minecraft.class_3218;
import net.minecraft.class_3949;
import net.minecraft.class_5268;
import net.minecraft.class_5321;
import net.minecraft.class_5574;
import net.minecraft.class_6880;
import net.minecraft.server.MinecraftServer;
import org.jetbrains.annotations.Nullable;
import org.spongepowered.asm.mixin.Final;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.Redirect;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

import java.util.Collections;
import java.util.List;
import java.util.concurrent.Executor;
import java.util.function.Consumer;

@Mixin(class_3218.class)
public abstract class MixinServerLevel extends MixinLevel implements IServerLevelMixin {
    @Shadow @Final private MinecraftServer server;
    //@Override
    //public PersistentEntitySectionManager<Entity> lmc$getEntityManager() {
   //     return this.entityManager;
   // }

    @Shadow
    public abstract class_26 getDataStorage();

    //@Shadow @Final private PersistentEntitySectionManager<Entity> entityManager;
    //? if >1.21.1 {
    /*@Override
    public boolean lmc$shouldDiscardEntity(Entity entity) {
        return false;
    }
    *///?} else if >1.16.5 {
    @Shadow protected abstract boolean shouldDiscardEntity(class_1297 arg);


    @Override
    public boolean lmc$shouldDiscardEntity(class_1297 entity) {
        return shouldDiscardEntity(entity);
    }
    //?} else {
    /*@Override
    public boolean lmc$shouldDiscardEntity(Entity entity) {
        return this.server.isSpawningAnimals() || !(entity instanceof Animal) && !(entity instanceof WaterAnimal)
                ? !this.server.areNpcsEnabled() && entity instanceof Npc
                : true;
    }
    *///?}

    @Unique
    private static final ObjectIterator<class_1297> lmc$emptyEntityIter = new ObjectIterator<class_1297>(){
        @Override
        public boolean hasNext() {
            return false;
        }

        @Override
        public class_1297 next() {
            return null;
        }
    };

    //? if >1.16.5 {
    @Redirect(method = "tick",at = @At(value = "INVOKE",target = "Lnet/minecraft/world/level/entity/EntityTickList;forEach(Ljava/util/function/Consumer;)V"))
    public void lmcEntitiesTickChunkwiseOverride(class_5574 instance, Consumer<class_1297> entity){
    //?} else {
    /*@Redirect(method = "tick",at = @At(remap = false,value = "INVOKE",target = "Lit/unimi/dsi/fastutil/objects/ObjectSet;iterator()Lit/unimi/dsi/fastutil/objects/ObjectIterator;"))
    public ObjectIterator<Entity> lmcEntitiesTickChunkwiseOverride(ObjectSet<Entity> instance){
    *///?}
        //noinspection resource
        if (loadMyChunks$cast().method_8398() instanceof class_3215) {
            class_3215 scc = (class_3215) loadMyChunks$cast().method_8398();
            Long2ObjectLinkedOpenHashMap<class_3193> updatingChunkMap = ((IChunkMapMixin) scc.field_17254).lmc$getUpdatingChunkMap();
            synchronized (updatingChunkMap) {
                for (class_3193 value : updatingChunkMap.values()) {
                    if (value != null && value.method_16144() instanceof ILevelChunkMixin) {
                        ILevelChunkMixin chunk = (ILevelChunkMixin) value.method_16144();
                        if (((IChunkMapMixin) scc.field_17254).lmc$inEntityTickingRange(value.method_13994().method_8324())) {
                            chunk.loadMyChunks$tickEntities(getProfiler());
                        }
                    }
                }
            }
            // Effectively skips the vanilla behavior.
            //? if <1.16.6 {
            /*return lmc$emptyEntityIter;
            *///?}
        }
        //? if <1.16.6 {
        /*return instance.iterator();
        *///?}
    }



    //? if >1.20.5 {
    /*@Inject(method = "<init>",at = @At("RETURN"))
    public void injectCustomSaveData(MinecraftServer minecraftServer, Executor executor, LevelStorageSource.LevelStorageAccess levelStorageAccess, ServerLevelData serverLevelData, ResourceKey<Level> resourceKey, LevelStem levelStem, ChunkProgressListener chunkProgressListener, boolean bl, long l, List<CustomSpawner> list, boolean bl2, @Nullable RandomSequences randomSequences, CallbackInfo ci){
        SavedData.Factory<ChunkDataManager.LevelChunkLoaderManager> factory = new SavedData.Factory<>(()->ChunkDataManager.getManager(ServerLevel.class.cast(this)),(tag,other)->ChunkDataManager.loadManager(ServerLevel.class.cast(this),tag), DataFixTypes.LEVEL);
        getDataStorage().computeIfAbsent(factory,"loadmychunks_manager");
    }
    *///?}

    //? if >1.20.3 && <=1.20.5 {
    /*@Inject(method = "<init>",at = @At("RETURN"))
    public void injectCustomSaveData(MinecraftServer minecraftServer, Executor executor, LevelStorageSource.LevelStorageAccess levelStorageAccess, ServerLevelData serverLevelData, ResourceKey resourceKey, LevelStem levelStem, ChunkProgressListener chunkProgressListener, boolean bl, long l, List list, boolean bl2, RandomSequences randomSequences, CallbackInfo ci){
        SavedData.Factory<ChunkDataManager.LevelChunkLoaderManager> factory = new SavedData.Factory<>(()->ChunkDataManager.getManager(ServerLevel.class.cast(this)), tag->ChunkDataManager.loadManager(ServerLevel.class.cast(this),tag), DataFixTypes.LEVEL);
        getDataStorage().computeIfAbsent(factory,"loadmychunks_manager");
    }
    *///?}

    //? if >1.19.4 && <=1.20.1 {
    /*@Inject(method = "<init>",at = @At("RETURN"))
    public void injectCustomSaveData(
            MinecraftServer minecraftServer, Executor executor, LevelStorageSource.LevelStorageAccess arg, ServerLevelData arg2, ResourceKey arg3, LevelStem arg4, ChunkProgressListener arg5, boolean bl, long l, List list, boolean bl2,
            RandomSequences arg6,
            CallbackInfo ci){
        getDataStorage().computeIfAbsent(tag->ChunkDataManager.loadManager(ServerLevel.class.cast(this),tag),()->ChunkDataManager.getManager(ServerLevel.class.cast(this)),"loadmychunks_manager");
    }
    *///?}



    //? if >1.18.2 && <=1.19.4 {
    /*@Inject(method = "<init>",at = @At("RETURN"))
    public void injectCustomSaveData(MinecraftServer minecraftServer, Executor executor, LevelStorageSource.LevelStorageAccess levelStorageAccess, ServerLevelData serverLevelData, ResourceKey resourceKey, LevelStem levelStem, ChunkProgressListener chunkProgressListener, boolean bl, long l, List list, boolean bl2, CallbackInfo ci){
        getDataStorage().computeIfAbsent(tag->ChunkDataManager.loadManager(ServerLevel.class.cast(this),tag),()->ChunkDataManager.getManager(ServerLevel.class.cast(this)),"loadmychunks_manager");
    }
    *///?}

    //? if >1.18.1 && <1.18.3 {
    @Inject(method = "<init>", at = @At("RETURN"))
    public void injectCustomSaveData(MinecraftServer minecraftServer, Executor executor, class_32.class_5143 levelStorageAccess, class_5268 serverLevelData, class_5321 resourceKey, class_6880 holder, class_3949 chunkProgressListener, class_2794 chunkGenerator, boolean bl, long l, List list, boolean bl2, CallbackInfo ci) {
        getDataStorage().method_17924((tag) -> ChunkDataManager.loadManager(class_3218.class.cast(this), tag), () -> ChunkDataManager.getManager(class_3218.class.cast(this)), "loadmychunks_manager");
    }
    //?}

    //? if >1.16.5 && <=1.18.1 {
    /*@Inject(method = "<init>",at = @At("RETURN"))
    public void injectCustomSaveData(MinecraftServer minecraftServer, Executor executor, LevelStorageSource.LevelStorageAccess levelStorageAccess, ServerLevelData serverLevelData, ResourceKey resourceKey, LevelStem levelStem, ChunkProgressListener chunkProgressListener, boolean bl, long l, List list, boolean bl2, RandomSequences randomSequences, CallbackInfo ci){
        getDataStorage().computeIfAbsent(tag->ChunkDataManager.loadManager(ServerLevel.class.cast(this),tag),()->ChunkDataManager.getManager(ServerLevel.class.cast(this)),"loadmychunks_manager");
    }
    *///?}

    //? if <=1.16.5 {
    /*@Inject(method = "<init>",at = @At("RETURN"))
    public void injectCustomSaveData(MinecraftServer minecraftServer, Executor executor, LevelStorageSource.LevelStorageAccess levelStorageAccess, ServerLevelData serverLevelData, ResourceKey resourceKey, DimensionType dimensionType, ChunkProgressListener chunkProgressListener, ChunkGenerator chunkGenerator, boolean bl, long l, List list, boolean bl2, CallbackInfo ci) {
        getDataStorage().computeIfAbsent(() -> ChunkDataManager.getManager(ServerLevel.class.cast(this)), "loadmychunks_manager");
    }
    *///?}
}

==> loadmychunks-1.2.1%2B1.21.2%2Bfabric-sources.jar: com/drathonix/loadmychunks/common/mixin/MixinServerLevel.java <==
package com.drathonix.loadmychunks.common.mixin;

import com.drathonix.loadmychunks.common.bridge.IChunkMapMixin;
import com.drathonix.loadmychunks.common.bridge.ILevelChunkMixin;
import com.drathonix.loadmychunks.common.bridge.IServerLevelMixin;
import com.drathonix.loadmychunks.common.system.ChunkDataManager;
//? if >1.18.1 {
//?}
import it.unimi.dsi.fastutil.longs.Long2ObjectLinkedOpenHashMap;
import it.unimi.dsi.fastutil.objects.ObjectIterator;
import it.unimi.dsi.fastutil.objects.ObjectSet;
import net.minecraft.class_1297;
import net.minecraft.class_18;
import net.minecraft.class_1937;
import net.minecraft.class_26;
import net.minecraft.class_3193;
import net.minecraft.class_32;
import net.minecraft.class_3215;
import net.minecraft.class_3218;
import net.minecraft.class_3949;
import net.minecraft.class_4284;
import net.minecraft.class_5268;
import net.minecraft.class_5304;
import net.minecraft.class_5321;
import net.minecraft.class_5363;
import net.minecraft.class_5574;
import net.minecraft.class_8565;
import net.minecraft.server.MinecraftServer;
import org.jetbrains.annotations.Nullable;
import org.spongepowered.asm.mixin.Final;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.Redirect;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

import java.util.Collections;
import java.util.List;
import java.util.concurrent.Executor;
import java.util.function.Consumer;

@Mixin(class_3218.class)
public abstract class MixinServerLevel extends MixinLevel implements IServerLevelMixin {
    @Shadow @Final private MinecraftServer server;
    //@Override
    //public PersistentEntitySectionManager<Entity> lmc$getEntityManager() {
   //     return this.entityManager;
   // }

    @Shadow
    public abstract class_26 getDataStorage();

    //@Shadow @Final private PersistentEntitySectionManager<Entity> entityManager;
    //? if >1.21.1 {
    @Override
    public boolean lmc$shouldDiscardEntity(class_1297 entity) {
        return false;
    }
    //?} else if >1.16.5 {
    /*@Shadow protected abstract boolean shouldDiscardEntity(Entity arg);


    @Override
    public boolean lmc$shouldDiscardEntity(Entity entity) {
        return shouldDiscardEntity(entity);
    }
    *///?} else {
    /*@Override
    public boolean lmc$shouldDiscardEntity(Entity entity) {
        return this.server.isSpawningAnimals() || !(entity instanceof Animal) && !(entity instanceof WaterAnimal)
                ? !this.server.areNpcsEnabled() && entity instanceof Npc
                : true;
    }
    *///?}

    @Unique
    private static final ObjectIterator<class_1297> lmc$emptyEntityIter = new ObjectIterator<class_1297>(){
        @Override
        public boolean hasNext() {
            return false;
        }

        @Override
        public class_1297 next() {
            return null;
        }
    };

    //? if >1.16.5 {
    @Redirect(method = "tick",at = @At(value = "INVOKE",target = "Lnet/minecraft/world/level/entity/EntityTickList;forEach(Ljava/util/function/Consumer;)V"))
    public void lmcEntitiesTickChunkwiseOverride(class_5574 instance, Consumer<class_1297> entity){
    //?} else {
    /*@Redirect(method = "tick",at = @At(remap = false,value = "INVOKE",target = "Lit/unimi/dsi/fastutil/objects/ObjectSet;iterator()Lit/unimi/dsi/fastutil/objects/ObjectIterator;"))
    public ObjectIterator<Entity> lmcEntitiesTickChunkwiseOverride(ObjectSet<Entity> instance){
    *///?}
        //noinspection resource
        if (loadMyChunks$cast().method_8398() instanceof class_3215) {
            class_3215 scc = (class_3215) loadMyChunks$cast().method_8398();
            Long2ObjectLinkedOpenHashMap<class_3193> updatingChunkMap = ((IChunkMapMixin) scc.field_17254).lmc$getUpdatingChunkMap();
            synchronized (updatingChunkMap) {
                for (class_3193 value : updatingChunkMap.values()) {
                    if (value != null && value.method_16144() instanceof ILevelChunkMixin) {
                        ILevelChunkMixin chunk = (ILevelChunkMixin) value.method_16144();
                        if (((IChunkMapMixin) scc.field_17254).lmc$inEntityTickingRange(value.method_60473().method_8324())) {
                            chunk.loadMyChunks$tickEntities(getProfiler());
                        }
                    }
                }
            }
            // Effectively skips the vanilla behavior.
            //? if <1.16.6 {
            /*return lmc$emptyEntityIter;
            *///?}
        }
        //? if <1.16.6 {
        /*return instance.iterator();
        *///?}
    }



    //? if >1.20.5 {
    @Inject(method = "<init>",at = @At("RETURN"))
    public void injectCustomSaveData(MinecraftServer minecraftServer, Executor executor, class_32.class_5143 levelStorageAccess, class_5268 serverLevelData, class_5321<class_1937> resourceKey, class_5363 levelStem, class_3949 chunkProgressListener, boolean bl, long l, List<class_5304> list, boolean bl2, @Nullable class_8565 randomSequences, CallbackInfo ci){
        class_18.class_8645<ChunkDataManager.LevelChunkLoaderManager> factory = new class_18.class_8645<>(()->ChunkDataManager.getManager(class_3218.class.cast(this)),(tag,other)->ChunkDataManager.loadManager(class_3218.class.cast(this),tag), class_4284.field_19212);
        getDataStorage().method_17924(factory,"loadmychunks_manager");
    }
    //?}

    //? if >1.20.3 && <=1.20.5 {
    /*@Inject(method = "<init>",at = @At("RETURN"))
    public void injectCustomSaveData(MinecraftServer minecraftServer, Executor executor, LevelStorageSource.LevelStorageAccess levelStorageAccess, ServerLevelData serverLevelData, ResourceKey resourceKey, LevelStem levelStem, ChunkProgressListener chunkProgressListener, boolean bl, long l, List list, boolean bl2, RandomSequences randomSequences, CallbackInfo ci){
        SavedData.Factory<ChunkDataManager.LevelChunkLoaderManager> factory = new SavedData.Factory<>(()->ChunkDataManager.getManager(ServerLevel.class.cast(this)), tag->ChunkDataManager.loadManager(ServerLevel.class.cast(this),tag), DataFixTypes.LEVEL);
        getDataStorage().computeIfAbsent(factory,"loadmychunks_manager");
    }
    *///?}

    //? if >1.19.4 && <=1.20.1 {
    /*@Inject(method = "<init>",at = @At("RETURN"))
    public void injectCustomSaveData(
            MinecraftServer minecraftServer, Executor executor, LevelStorageSource.LevelStorageAccess arg, ServerLevelData arg2, ResourceKey arg3, LevelStem arg4, ChunkProgressListener arg5, boolean bl, long l, List list, boolean bl2,
            RandomSequences arg6,
            CallbackInfo ci){
        getDataStorage().computeIfAbsent(tag->ChunkDataManager.loadManager(ServerLevel.class.cast(this),tag),()->ChunkDataManager.getManager(ServerLevel.class.cast(this)),"loadmychunks_manager");
    }
    *///?}



    //? if >1.18.2 && <=1.19.4 {
    /*@Inject(method = "<init>",at = @At("RETURN"))
    public void injectCustomSaveData(MinecraftServer minecraftServer, Executor executor, LevelStorageSource.LevelStorageAccess levelStorageAccess, ServerLevelData serverLevelData, ResourceKey resourceKey, LevelStem levelStem, ChunkProgressListener chunkProgressListener, boolean bl, long l, List list, boolean bl2, CallbackInfo ci){
        getDataStorage().computeIfAbsent(tag->ChunkDataManager.loadManager(ServerLevel.class.cast(this),tag),()->ChunkDataManager.getManager(ServerLevel.class.cast(this)),"loadmychunks_manager");
    }
    *///?}

    //? if >1.18.1 && <1.18.3 {
    /*@Inject(method = "<init>", at = @At("RETURN"))
    public void injectCustomSaveData(MinecraftServer minecraftServer, Executor executor, LevelStorageSource.LevelStorageAccess levelStorageAccess, ServerLevelData serverLevelData, ResourceKey resourceKey, Holder holder, ChunkProgressListener chunkProgressListener, ChunkGenerator chunkGenerator, boolean bl, long l, List list, boolean bl2, CallbackInfo ci) {
        getDataStorage().computeIfAbsent((tag) -> ChunkDataManager.loadManager(ServerLevel.class.cast(this), tag), () -> ChunkDataManager.getManager(ServerLevel.class.cast(this)), "loadmychunks_manager");
    }
    *///?}

    //? if >1.16.5 && <=1.18.1 {
    /*@Inject(method = "<init>",at = @At("RETURN"))
    public void injectCustomSaveData(MinecraftServer minecraftServer, Executor executor, LevelStorageSource.LevelStorageAccess levelStorageAccess, ServerLevelData serverLevelData, ResourceKey resourceKey, LevelStem levelStem, ChunkProgressListener chunkProgressListener, boolean bl, long l, List list, boolean bl2, RandomSequences randomSequences, CallbackInfo ci){
        getDataStorage().computeIfAbsent(tag->ChunkDataManager.loadManager(ServerLevel.class.cast(this),tag),()->ChunkDataManager.getManager(ServerLevel.class.cast(this)),"loadmychunks_manager");
    }
    *///?}

    //? if <=1.16.5 {
    /*@Inject(method = "<init>",at = @At("RETURN"))
    public void injectCustomSaveData(MinecraftServer minecraftServer, Executor executor, LevelStorageSource.LevelStorageAccess levelStorageAccess, ServerLevelData serverLevelData, ResourceKey resourceKey, DimensionType dimensionType, ChunkProgressListener chunkProgressListener, ChunkGenerator chunkGenerator, boolean bl, long l, List list, boolean bl2, CallbackInfo ci) {
        getDataStorage().computeIfAbsent(() -> ChunkDataManager.getManager(ServerLevel.class.cast(this)), "loadmychunks_manager");
    }
    *///?}
}
